/*
 * Copyright (c) 2002-2009, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.workflow.business.task;

import fr.paris.lutece.portal.service.util.AppLogService;

import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;


/**
 * TaskFactory
 */
public class TaskFactory implements ITaskFactory
{
    private static Map<String, ITaskType> _mapTaskType;

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.workflow.business.task.ITaskFactory#setTaskTypeMap(java.util.Map)
     */
    public void setTaskTypeMap( Map<String, ITaskType> mapTaskType )
    {
        _mapTaskType = mapTaskType;

        for ( Object entryMap : _mapTaskType.entrySet(  ) )
        {
            ( (Entry<String, ITaskType>) entryMap ).getValue(  )
              .setKey( ( (Entry<String, ITaskType>) entryMap ).getKey(  ) );
        }
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.workflow.business.task.ITaskFactory#getTask(java.lang.String)
     */
    public ITask getTask( String strKey )
    {
        ITask task = null;

        if ( _mapTaskType != null )
        {
            ITaskType taskType = _mapTaskType.get( strKey );

            if ( taskType != null )
            {
                try
                {
                    task = (ITask) Class.forName( taskType.getClassName(  ) ).newInstance(  );
                    task.setTaskType( taskType );
                }
                catch ( ClassNotFoundException e )
                {
                    // class doesn't exist
                    AppLogService.error( e );
                }
                catch ( InstantiationException e )
                {
                    // Class is abstract or is an interface or haven't accessible
                    // builder
                    AppLogService.error( e );
                }
                catch ( IllegalAccessException e )
                {
                    // can't access to rhe class
                    AppLogService.error( e );
                }
            }
        }

        return task;
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.workflow.business.task.ITaskFactory#getAllTaskType()
     */
    public Collection<ITaskType> getAllTaskType(  )
    {
        if ( _mapTaskType != null )
        {
            return _mapTaskType.values(  );
        }

        return null;
    }
}
